#!/usr/bin/env python

# NTFY API
# specification: https://docs.ntfy.sh/subscribe/api/

# original:
# https://github.com/iacchus/python-pushover-open-client/blob/main/src/python_pushover_open_client/__init__.py

import base64
import os
import pathlib
import shlex
import shutil
import sys
import types

import requests
import websocket

from importlib.metadata import PackageNotFoundError, version  # pragma: no cover


FUNCTION = types.FunctionType

#  DEBUG: bool = False
DEBUG: bool = True

if DEBUG:
    websocket.enableTrace(True)

# from importlib.metadata import PackageNotFoundError, version  # pragma: no cover

# if sys.version_info[:2] >= (3, 8):
#    # TODO: Import directly (no need for conditional) when `python_requires = >= 3.8`
#    from importlib.metadata import PackageNotFoundError, version  # pragma: no cover
# else:
#    from importlib_metadata import PackageNotFoundError, version  # pragma: no cover

try:
    # Change here if project is renamed and does not equal the package name
    dist_name = "ntfy-client"
    __version__ = version(dist_name)
except PackageNotFoundError:  # pragma: no cover
    __version__ = "unknown"
finally:
    del version, PackageNotFoundError

PRIORITIES = {"urgent", "high", "default", "low", "min"}
YES_NO = {"yes", "no"}
ON_OFF = {"1", "0"}

FILE_RECEIVED_MESSAGE = "You received a file: {file_name}"

default_config_dict = {
    #  "hostname":  NTFY_SERVER_HOSTNAME,
    #  "topic": NTFY_TOPIC,
    #  "token": NTFY_TOKEN,

    "title": "from ntfycli.py",
    "message": "defmsg",
    "priority": "default",
    "tags": [],
    "markdown": None,
    "delay": None,
    "actions": None,
    "click": None,
    "attach": None,
    "filename": None,
    "email": None,
    "call": None,

    "cache": None,
    "firebase": None,
    "unifiedpush": None,

    "authorization": None,
    "content_type": None
    }


# FIXME:
# Group mutually exclusive options (argparse docs)
# those are:
# * X-UnifiedPush vs X-Firebase
# * Authorization Basic vs Bearer
# * Authorization Username (envvar/arg) vs Bearer auth
# * Authorization Password (envvar/arg) vs Bearer
# * Authorization Token (envvar/arg) vs Basic auth
# * X-Attach vs X-File
# * X-Markdown vs Content-Type != "text/markdown"

# FIXME: UnifiedPush vs --file sending file as message

# TODO; sanitize parameters?

# TODO: CONFIG PRIORITY
# (first in this list has greater priority)
# 
# command-line arguments (user)
# environment variables (user, file or inline)
# ~/.notify-cli.conf (user, file)
# /etc/notify-cli.conf (system, file)
# (script, hardcoded defaults)


class NTFYClient:

    hostname: str
    topic: str
    token: str

    title: str
    message: str
    priority: str
    tags: list[str]  # ", ".join(tags)
    markdown: bool
    delay: str
    actions: list[str]  # "; ".join(actions)
    click: str
    attach: str
    filename: str
    email: str
    call: str

    cache: str
    firebase: str
    unifiedpush: str

    authorization: str | tuple[str, str]
    content_type: str

    def __init__(self, hostname, topic, token, args):
        self.hostname = hostname
        self.topic = topic
        self.token = token

        self.message = args.message or sys.stdin  # FIXME
        self.args = args  # FIXME: remove this: process it yourself

        self.url_https = f"https://{hostname}/{topic}"
        self.url_wss = f"wss://{hostname}/{topic}/ws"

        self.authorization = self.make_auth_token(method="basic")
        self.headers = self.make_headers(args)

    def pub(self, message=None):
        # FIXME: message above
        file_path = pathlib.Path(self.args.file)

        if file_path.is_file():
            self.headers.update({
                "X-Title": "File received (via ntfy)",
                "X-Message": FILE_RECEIVED_MESSAGE.format(file_name=self.args.filename
                                                          or file_path.name),
                "X-tags": "gift",
                "X-Filename": self.args.filename or file_path.name
                })
        elif self.args.file and not file_path.is_file():
            print(f"Error: file '{file_path.absolute()}' not found, exiting.")
            exit(1)

        data = open(file_path.absolute(), "rb") \
                if self.args.file else self.args.message

        response = requests.post(url=self.url_https,
                                 data=data,
                                 headers=self.headers)

        return response

    def make_headers(self, args):
        all_headers = {
                #  "X-Title": args.title or DEFAULT_MESSAGE_TITLE,
                "X-Title": args.title,
                "X-Icon": args.icon or None, #ICON_IMAGE_URL,
                "X-Priority": args.priority or None,
                "X-Tags": ",".join(args.tags if args.tags else []),
                "X-Markdown": args.markdown or None,
                "X-Delay": args.delay or None,
                "X-Actions": "; ".join(args.actions if args.actions else []),
                "X-Click": args.click or None,
                "X-Attach": args.attach,
                "X-Filename": args.filename or None,
                "X-Email": args.email or None,
                "X-Call": args.call or None,
                "X-Cache": args.cache or None,
                "X-Firebase": args.firebase or None,
                "X-UnifiedPush": args.unifiedpush or None,
                "X-Poll-ID": args.poll_id or None,
                "Authorization": self.authorization,
                "Content-Type": args.content_type or None,
                }

        headers = {key: value for key, value in all_headers.items() if value}
        print("all_headers", all_headers, end="\n\n")
        print("headers", headers, end="\n\n")

        return headers

    def make_auth_token(self, method="basic"):
        if method == "basic":
            auth_string = f":{self.token}"  # str: "empty_username:token"
            auth_string_bytes = auth_string.encode("ascii")
            auth_string_base64 = base64.b64encode(auth_string_bytes).decode("utf-8")

            authorization = f"Basic {auth_string_base64}"
        elif method == "query_param":
            # FIXME: deduplicate this code with the above
            auth_string = f":{self.token}"  # str: "empty_username:token"
            auth_string_bytes = auth_string.encode("ascii")
            auth_header_basic = \
                    base64.b64encode(auth_string_bytes).decode("utf-8")
            auth_string_base64 = base64.b64encode(auth_string_bytes).decode("utf-8")

            authorization = f"Basic {auth_string_base64}"
            auth_header_query_param_key = "auth"
            auth_header_query_param_value = \
                    base64.b64encode(auth_header_basic
                                     .encode("ascii")).decode("utf-8")
            authorization = (auth_header_query_param_key,
                             auth_header_query_param_value)
        else:
            authorization = f"Bearer {self.token}"

        return authorization




import json
import types

import click

from . import NTFYClient


server_hostname_option = \
        click.option("--server-hostname",
                     envvar="NTFY_SERVER_HOSTNAME")
topic_option = \
        click.option("--topic",
                     envvar="NTFY_TOPIC")
token_option = \
        click.option("--token",
                     envvar="NTFY_TOKEN")
message_option = \
        click.option("--message",
                     envvar="NTFY_DEFAULT_MESSAGE")
title_option = \
        click.option("--title",
                     envvar="NTFY_DEFAULT_TITLE")
priority_option = \
        click.option("--priority",
                     envvar="NTFY_PRIORITY")
tags_option = \
        click.option("--tags", "--tag",
                     multiple=True,
                     envvar="NTFY_TAGS")
delay_option = \
        click.option("--delay",
                     envvar="NTFY_DELAY")
actions_option = \
        click.option("--actions",
                     envvar="NTFY_ACTIONS")
click_option = \
        click.option("--click",
                     envvar="NTFY_CLICK")
markdown_option = \
        click.option("--markdown",
                     envvar="NTFY_MARKDOWN")
icon_option = \
        click.option("--icon",
                     envvar="NTFY_ICON")
file_option = \
        click.option("--file",
                     type=str,
                     default="",
                     envvar="NTFY_FILE")
attach_option = \
        click.option("--attach",
                     envvar="NTFY_ATTACH")
firebase_option = \
        click.option("--firebase",
                     envvar="NTFY_FIREBASE")
unifiedpush_option = \
        click.option("--unifiedpush",
                     envvar="NTFY_UNIFIEDPUSH")
filename_option = \
        click.option("--filename",
                     envvar="NTFY_FILENAME")
email_option = \
        click.option("--email",
                     envvar="NTFY_EMAIL")
call_option = \
        click.option("--call",
                     envvar="NTFY_CALL")
cache_option = \
        click.option("--cache",
                     envvar="NTFY_CACHE")
poll_id_option = \
        click.option("--poll-id",
                     envvar="NTFY_POLL_ID")
content_type_option = \
        click.option("--content-type",
                     envvar="NTFY_CONTENT_TYPE")

@click.group
def cli():
    pass

@cli.command()
@server_hostname_option
@topic_option
@token_option
@message_option
@title_option
@priority_option
@tags_option
@delay_option
@actions_option
@click_option
@markdown_option
@icon_option
@file_option
@attach_option
@firebase_option
@unifiedpush_option
@filename_option
@email_option
@call_option
@cache_option
@poll_id_option
@content_type_option
def pub(server_hostname, topic, token, *args, **kwargs):

    args = types.SimpleNamespace(**kwargs)
    print("ARGS", args, end="\n\n")
    print("KWARGS", kwargs, end="\n\n")


    ntfy_client = NTFYClient(hostname=server_hostname,
                            topic=topic,
                            token=token,
                            args=args)

    r = ntfy_client.pub()
    response_data = json.dumps(r.json(), indent=2)
    print("Response data:", response_data, sep="\n")

if __name__ == "__main__":

    cli()

